import 'package:flutter/services.dart';
import 'package:flutter/foundation.dart' as foundation;

class FlutterBaseBannerRefined {
  FlutterBaseBannerRefined({
    required String channelNm,
    required String zoneId,
  }) {
    this.zoneId = zoneId;
    this.channelName = channelNm;
    this.key = foundation.UniqueKey().toString();

    _initChannel(channelNm);
  }

  void _initChannel(String channelName) async {
    platform = MethodChannel(channelName);
    platform.setMethodCallHandler(_handleMethod);
    platform.invokeMethod("makeAdView", {'key': key, 'zoneId': zoneId});
  }

  late String zoneId;
  late String channelName;
  late MethodChannel platform;
  late String key;

  Function()? onLoadAdCallback;
  Function(String errorMsg)? onFailAdCallback;

  void load() async {
    platform.invokeMethod('load');
  }

  void showBanner() {
    platform.invokeMethod('showBanner');
  }

  void hideBanner() {
    platform.invokeMethod('hideBanner');
  }

  void removeBanner() {
    platform.invokeMethod('removeBanner');
  }

  Future<dynamic> _handleMethod(MethodCall call) async {
    switch (call.method) {
      case 'onLoadAd':
        if (onLoadAdCallback != null) {
          onLoadAdCallback!();
        }
        break;
      case 'onFailAd':
        if (onFailAdCallback != null) {
          onFailAdCallback!(call.arguments.toString());
        }
        break;
      default:
        throw ('method not defined');
    }
  }

  void setCallbackListener({
    void Function()? onLoadAd,
    void Function(String errorMsg)? onFailAd,
  }) {
    onLoadAdCallback = onLoadAd;
    onFailAdCallback = onFailAd;
  }
}
