import 'package:flutter/foundation.dart' as foundation;
import 'package:flutter/services.dart';

class FlutterBidmadCommon {
  FlutterBidmadCommon._privateConstructor() {
    _initChannel(_DEFAULT_CHANNELNAME);
  }

  static final FlutterBidmadCommon _instance =
      FlutterBidmadCommon._privateConstructor();

  factory FlutterBidmadCommon() {
    return _instance;
  }

  static const _DEFAULT_CHANNELNAME = "com.bidmad.sdk.common/Flutter";
  late MethodChannel _platform;

  Function(bool isInitialized)? onInitialized;
  Function(bool isAdFree)? onAdStatus;

  Future<void> _initChannel(String channelNm) async {
    _platform = MethodChannel(channelNm);
    _platform.setMethodCallHandler(_handleMethod);
  }

  Future<void> setDebugging(bool isDebug) async {
    return await _platform.invokeMethod('setDebugging', {"isDebug": isDebug});
  }

  Future<String> initBannerChannel() async {
    String chanNm = await _platform.invokeMethod('initBannerChannel');
    return chanNm;
  }

  Future<void> initBannerWidget() async {
    return await _platform.invokeMethod('initBannerWidget');
  }

  Future<String> initBannerRefinedChannel() async {
    String chanNm = await _platform.invokeMethod('initBannerRefinedChannel');
    return chanNm;
  }

  Future<void> initNativeAdWidget(String layoutName) async {
    return await _platform.invokeMethod('initNativeAdWidget', {"layoutName": layoutName});
  }

  Future<String> initInterstitialChannel() async {
    String chanNm = await _platform.invokeMethod('initInterstitialChannel');
    return chanNm;
  }

  Future<String> initRewardChannel() async {
    String chanNm = await _platform.invokeMethod('initRewardChannel');
    return chanNm;
  }

  Future<String> initGDPRChannel() async {
    String chanNm = await _platform.invokeMethod('initGDPRforGoogle');
    return chanNm;
  }

  Future<void> setUseArea(bool useArea) async {
    return await _platform.invokeMethod('setUseArea', useArea);
  }

  Future<void> setGDPRSetting(bool consent) async {
    return await _platform.invokeMethod('setGDPRSetting', consent);
  }

  Future<int> getGDPRSetting() async {
    int consentStatus = await _platform.invokeMethod('getGDPRSetting');
    return consentStatus;
  }

  Future<void> initializeSdk(String appKey) async {
    return await _platform.invokeMethod('initializeSdk', {"appKey": appKey});
  }

  Future<void> initializeSdkWithCallback(String appKey) async {
    return await _platform
        .invokeMethod('initializeSdkWithCallback', {"appKey": appKey});
  }

  void setInitializeCallbackListener({
    void Function(bool isInitialized)? onInitialized,
  }) {
    this.onInitialized = onInitialized;
  }

  Future<void> setAdFreeEventListener({
    void Function(bool isAdFree) ? onAdStatus,
  }) async {
    this.onAdStatus = onAdStatus;
    return await _platform.invokeMethod('setAdFreeEventListener');
  }

  Future<String> reqAdTrackingAuthorization() async {

    if (foundation.defaultTargetPlatform == foundation.TargetPlatform.iOS) {
      String attResponse = await _platform.invokeMethod('reqAdTrackingAuthorization');
      return attResponse;
    }

    return "Available Only for iOS";
  }

  Future<void> setAdvertiserTrackingEnabled(bool enable) async {
    if (foundation.defaultTargetPlatform == foundation.TargetPlatform.iOS) {
      return await _platform.invokeMethod('setAdvertiserTrackingEnabled', enable);
    }
  }

  Future<bool> getAdvertiserTrackingEnabled() async {
    if (foundation.defaultTargetPlatform == foundation.TargetPlatform.iOS) {
      bool attEnabled = await _platform.invokeMethod('getAdvertiserTrackingEnabled');
      return attEnabled;
    }

    return false;
  }

  Future<void> setCUID(String cuid) async {
    return await _platform.invokeMethod('setCUID', cuid);
  }

  Future<bool> isAdFree() async {
    return await _platform.invokeMethod('isAdFree');
  }

  Future<dynamic> _handleMethod(MethodCall call) async {
    switch (call.method) {
      case 'onInitialized':
        if (onInitialized != null) {
          onInitialized!(call.arguments as bool);
        }
        break;
      case 'onAdStatus' :
        if(onAdStatus != null){
          onAdStatus!(call.arguments as bool);
        }
        break;
      default:
        throw (call.method+'method not defined');
    }
  }
}
